<?php
/**
 * @package     Joomla.Plugin
 * @subpackage  System.imgtopicture
 *
 * @copyright   Copyright (C) 2005 - 2020 jtotal.org
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('JPATH_BASE') or die;



// include the syndicate functions only once
//require_once dirname(__FILE__) . '/helper.php';

JLoader::register('plgSystemImgtopictureHelper', __DIR__ . '/helper.php');

/**
 * System plugin.
 *
 * @since  2.5
 */
class PlgSystemImgtopicture extends JPlugin
{


	/**
	 * Method to catch the onAfterDispatch event.
	 *
	 * This is where we setup the click-through content highlighting for.
	 * The highlighting is done with JavaScript so we just
	 * need to check a few parameters and the JHtml behavior will do the rest.
	 *
	 * @return  boolean  True on success
	 *
	 * @since   2.5
	 */

	public static $debug_msg = array();
	public static $debug = false;
	public static $debug_images = false;
	public static $startmicro ;
	public static $webp = false;
	public static $webp_quality = 70;
	public static $jpg_quality = 80;
	public static $png_quality = 6;
	public static $itp_lazy = false;
	public static $itp_scale_all = false;
	public static $img_original_path;

	public function onAfterDispatch()
	{
		// Check that we are in the site application.
		if (JFactory::getApplication()->isAdmin())
		{
			return true;
		}

		// Set the variables.
		$input = JFactory::getApplication()->input;
		$extension = $input->get('option', '', 'cmd');


		if (JFactory::getDocument()->getType() !== 'html' || $input->get('tmpl', '', 'cmd') === 'component')
		{
			return true;
		}


		return true;
	}

	//ARTICLE
	public function onContentPrepare($context, &$article, &$params, $page = 0)
	{



        $input = JFactory::getApplication()->input;
        
        if($input->get('format', '')=='json') return;
	
		if(function_exists('gd_info') === false){
			return;
			$gdInfos = array('FATAL'=>'GD is not supported by your server');
		}else{
			$gdInfos = gd_info();
		}



		self::$startmicro = microtime(TRUE);
		

		$plugin = JPluginHelper::getPlugin('system', 'imgtopicture');
		$plugin_params = new JRegistry($plugin->params);

		self::$debug 		= $plugin_params->get('itp_debug','0');
		self::$webp_quality = $plugin_params->get('itp_webp_quality','70');
		self::$jpg_quality 	= $plugin_params->get('itp_jpg_quality','80');
		self::$png_quality 	= $plugin_params->get('itp_png_quality','6');
		self::$debug_images = $plugin_params->get('itp_debug_images','0');
		self::$itp_lazy 	= $plugin_params->get('itp_lazy','0');

		

		//LAZY
		$doc = JFactory::getDocument();
		//$doc->addScript(JURI::base() . 'plugins/system/imgtopicture/assets/ls.blur-up.min.js');


		if(self::$itp_lazy)	$doc->addScript(JURI::base() . 'plugins/system/imgtopicture/assets/lazysizes.min.js', array(), array('defer'=>'defer'));


		$article_img = $plugin_params->get('itp_article_images','0');
		$art_min_width = $plugin_params->get('itp_article_min_image','600');


		
		$browser = JBrowser::getInstance();

		//CHECK WEBP
		self::checkwebp();

		 

		//CHECK CONTEXT
		if(!plgSystemImgtopictureHelper::checkContext($this->params,$context)) return;


		//CHECK PAGES
		if(!plgSystemImgtopictureHelper::checkPages_v3($this->params,$article,$context)) return;




		//PREG_MATCH
 		if($article_img){
			//$regex		= '/<img(.*?)>/i';
			$regex		= "/\<img (.*?)\/>/i";

			//ARTICLE IMAGE
			preg_match_all($regex, $article->text, $matches, PREG_SET_ORDER);

			if ($matches){
				foreach ($matches as $match)
				{
					

					
					$matcheslist = explode(',', $match[0]);
					$found = trim($match[0]);



					if($this->check_minwidth($plugin_params,$found,"article")){
						self::addDebug("onContentPrepare check_minwidth",'false');
						continue;
					}

					//************ */
					//check by class

					$inc_exc_by_class =  $plugin_params->get('inc_exc_by_class','in');
					$in_ex_class =  $plugin_params->get('in_ex_class','','STRING');

					$hasClass = false;
					//check by class
					if($in_ex_class != ''){

						//class
						preg_match_all('/class\s*=\s*"([^"]+)"/i', $found, $class_regex);
						if(!empty($class_regex[0])){
							$classes = explode(' ',$class_regex[1][0]);
							foreach($classes as $key =>$class){
								if($class == $in_ex_class){
									$hasClass = true;
								}
							}
						}

						if($inc_exc_by_class == 'in'){
							if($hasClass == false) continue;
						}else{
							if($hasClass) continue;
						}
					}


					$thumbs_paths = $this->mainthumbs($plugin_params,$found);

					if($thumbs_paths != false){
						$article->text = str_replace($match[0], $thumbs_paths, $article->text);
					}

				}
			}
		} 


	}

	
	function onBeforeCompileHead()
	{

	}


	function checkWebp(){
		//CHECK IF IMAGEWEB creation is available
		$plugin = JPluginHelper::getPlugin('system', 'imgtopicture');
		$plugin_params = new JRegistry($plugin->params);
		if($plugin_params->get('itp_webp','0')){
			self::$webp = true;
			//check servers php
			if(function_exists('imagewebp') === false){
                self::$webp = false;
                self::addDebug("imagewebp function",'false');
			}else{
                $browser = JBrowser::getInstance();
				//check clients browser
				if( strpos( $_SERVER['HTTP_ACCEPT'], 'image/webp' ) !== false || 
					strpos( $_SERVER['HTTP_USER_AGENT'], ' Chrome/' ) !== false || 
					(strpos( $_SERVER['HTTP_USER_AGENT'], ' Firefox/' ) !== false && $browser->getMajor() >='66')) {
					// webp is supported!
					self::$webp = true;
					self::addDebug("webp browser",'supported');
				}else{
					self::$webp = false;
					self::addDebug("webp browser",'NOT supported');
				}
			}
		}
		//check webp support. If not, use scale_all by default (to create files version in jpg)
		if(self::$webp){
			self::$itp_scale_all= $plugin_params->get('itp_scale_all','0');
		}else{
			self::$itp_scale_all = true;
		}
	}


	/**
	* //FULL IMAGE | INTRO IMAGE
	*/
	function onAfterRender()
	{


		//ONLY IN HTML
		$input = JFactory::getApplication()->input;
		if (JFactory::getDocument()->getType() !== 'html' || $input->get('tmpl', '', 'cmd') === 'component')
		{
			return true;
		}



		$plugin = JPluginHelper::getPlugin('system', 'imgtopicture');
		$plugin_params = new JRegistry($plugin->params);

		$intro_img = $plugin_params->get('itp_intro_images','0');
		$full_img = $plugin_params->get('itp_full_images','0');
		$art_min_width = $plugin_params->get('itp_article_min_image','600');


		self::$debug 		= $plugin_params->get('itp_debug','0');
		self::$webp_quality = $plugin_params->get('itp_webp_quality','70');
		self::$jpg_quality 	= $plugin_params->get('itp_jpg_quality','80');
		self::$png_quality 	= $plugin_params->get('itp_png_quality','6');


		self::$debug_images = $plugin_params->get('itp_debug_images','0');
		
		self::$itp_lazy 	= $plugin_params->get('itp_lazy','0');

		if($intro_img || $full_img){
			$buffer = JResponse::getBody();
		}

		//CHECK WEBP
		self::checkwebp();



		//FULL IMAGE
		if($full_img){

			//$regex	= '/\<img class=*"[^"]*"(.*?)\>/i';
			//$regex	= '/<img.+?class="caption(.*?)itemprop="image"+>/i';
			//$regex	= '|<img(.*?)>|';
			//$regex	= '/<img[^>]+src="([^">]+)"/';
			//$regex		= '/<img\s+[^>]*class="caption"[^>]*src="([^"]*)"[^>]*itemprop="image"[^>]*>/';
			//v1.1.2
			$regex		= '/<img\s+[^>]*src="([^"]*)"[^>]*itemprop="image"[^>]*>/';

			preg_match_all($regex, $buffer, $matches, PREG_SET_ORDER);

			if ($matches){
				foreach ($matches as $match)
				{
					$matcheslist = explode(',', $match[0]);
					$found = trim($match[0]);



					if($this->check_minwidth($plugin_params,$found,"full")){
						continue;
					}


					$thumbs_paths = $this->mainthumbs($plugin_params,$found);


					if($thumbs_paths != false){

						$buffer = str_replace($match[0], $thumbs_paths, $buffer);
					}
					//$article->text = str_replace($match[0], $output, $article->text);
				}
				//si hay coincidencias se insertan
				JResponse::setBody($buffer);
			}
		}


		//INTRO IMAGE
		if($intro_img){
			//$regex		= '/\<img class=*"[^"]*"(.*?)\>/i';
			//$regex		= '/<img.+?class="caption(.*?)itemprop="image"+>/i';
			//$regex		= '|<img(.*?)>|';
			//$regex		= '/<img[^>]+src="([^">]+)"/';
			$regex		= '/<img\s+[^>]*src="([^"]*)"[^>]*itemprop="thumbnailUrl"[^>]*>/';

			preg_match_all($regex, $buffer, $matches, PREG_SET_ORDER);

			if ($matches){
				foreach ($matches as $match)
				{
					$matcheslist = explode(',', $match[0]);
					$found = trim($match[0]);

					if($this->check_minwidth($plugin_params,$found,"intro")){
						continue;
					}

					$thumbs_paths = $this->mainthumbs($plugin_params,$found);
					
					//echo $match[0];
					if($thumbs_paths != false){
						$buffer 	= str_replace($match[0], $thumbs_paths, $buffer);
					}
					//$article->text = str_replace($match[0], $output, $article->text);
				}
				//si hay coincidencias se insertan
				JResponse::setBody($buffer);
			}
		}



	}


	protected function check_minwidth($plugin_params,$found,$mode="article"){

		if($mode == "article" ){
			$min_width = $plugin_params->get('itp_article_min_image','600', 'UINT');
		}elseif($mode == "intro"){
			$min_width = $plugin_params->get('itp_intro_min_image','600', 'UINT');

		}elseif($mode == "full"){
			$min_width = $plugin_params->get('itp_full_min_image','600', 'UINT');
		}else{
			return true;
		}
		
		preg_match_all('/src\s*=\s*"([^"]+)"/i', $found, $src_regex);
		$img_original = $src_regex[1][0];

		//CHECK EXTENSIONS

		$supported_image = array('gif','jpg','jpeg','png','webp','bmp','svg');
	 
		$ext = strtolower(pathinfo($img_original, PATHINFO_EXTENSION));


 
		if (in_array($ext, $supported_image)){

	 	} else  {
	 		return true;
		 }

		$actual_size_array = getimagesize($img_original);

		$actual_size_w = (int)$actual_size_array[0];

		if($min_width >= $actual_size_w){
			return true;
		}

		return false;
	}



	/**
	 * MAINTHUMBS
	 *
	 * @param   array  $plugin_params 
	 * @param   string  $found the image match
	 *
	 * @return  mixed
	 *
	 * @since   1.6
	 */
	protected function mainthumbs($plugin_params,$found)
	{


		//temportal
		//$thumbs_path = $params->get('thumbs_path', 'dx_visualmenu_thumbs');
		$thumbs_path = 'imagetopicture';
		//creo el directorio si no esta creado
		$ruta_thumb = JPATH_BASE.'/cache/'.$thumbs_path;

		
		plgSystemImgtopictureHelper::_createPath($ruta_thumb);

		/************************************/
		/*********  EXTRACCION  *************/
		/************************************/

		//TODO simplificar esto
		//preg_match_all('/(alt|title|src|class)\s*=\s*"([^"]+)"/i', $found, $lang_regex);

		//extraccion
			//ruta
			preg_match_all('/src\s*=\s*"([^"]+)"/i', $found, $src_regex);
			$img_original = $src_regex[1][0];

			self::$img_original_path = $img_original;

			
						
			//alt
			preg_match_all('/alt\s*=\s*"([^"]+)"/i', $found, $alt_regex);

			if(!empty($alt_regex[0])){
				$alt = 'alt="'.$alt_regex[1][0].'"';
			}else{
				$alt = "";
			}

			//id
			preg_match_all('/id\s*=\s*"([^"]+)"/i', $found, $id_regex);

			if(!empty($id_regex[0])){
				$id = 'id="'.$id_regex[1][0].'"';
			}else{
				$id = "";
			}

			//class
			preg_match_all('/class\s*=\s*"([^"]+)"/i', $found, $class_regex);

			if(!empty($class_regex[0])){
				$class = $class_regex[1][0];
			}else{
				$class = "";
			}

			//title
			preg_match_all('/title\s*=\s*"([^"]+)"/i', $found, $title_regex);

			if(!empty($title_regex[0])){
				$title = 'title="'.$title_regex[1][0].'"';
			}else{
				$title = "";
			}

			//lang
			preg_match_all('/lang\s*=\s*"([^"]+)"/i', $found, $lang_regex);

			if(!empty($lang_regex[0])){
				$lang = 'lang="'.$lang_regex[1][0].'"';
			}else{
				$lang = "";
			}

			//itemprop
			preg_match_all('/itemprop\s*=\s*"([^"]+)"/i', $found, $itemprop_regex);

			if(!empty($itemprop_regex[0])){
				$itemprop = 'itemprop="'.$itemprop_regex[1][0].'"';
			}else{
				$itemprop = "";
			}

			//dir
			preg_match_all('/dir\s*=\s*"([^"]+)"/i', $found, $dir_regex);

			if(!empty($dir_regex[0])){
				$dir = 'dir="'.$dir_regex[1][0].'"';
			}else{
				$dir = "";
			}

			//longdesc
			preg_match_all('/dir\s*=\s*"([^"]+)"/i', $found, $longdesc_regex);

			if(!empty($longdesc_regex[0])){
				$longdesc = 'longdesc="'.$longdesc_regex[1][0].'"';
			}else{
				$longdesc = "";
			}

			//styles
			preg_match_all('/style\s*=\s*"([^"]+)"/i', $found, $style_regex);

			if(!empty($style_regex[0])){
				$style = 'style="'.$style_regex[1][0].'"';
			}else{
				$style = "";
			}

			//width
			preg_match_all('/width\s*=\s*"([^"]+)"/i', $found, $iwidth_regex);

			if(!empty($iwidth_regex[0])){
				$iwidth = 'width="'.$iwidth_regex[1][0].'"';
			}else{
				$iwidth = "";
			}

			//height
			preg_match_all('/height\s*=\s*"([^"]+)"/i', $found, $iheight_regex);

			if(!empty($iheight_regex[0])){
				$iheight = 'height="'.$iheight_regex[1][0].'"';
			}else{
				$iheight = "";
			}


		//CREATE THUMBS
		$itp_lazy_low		= $plugin_params->get('$itp_lazy_low','64');

		$itp_mobile 		= $plugin_params->get('itp_mobile','0');
		$itp_mobile_w 		= $plugin_params->get('itp_mobile_width','300');
		$itp_mobile_retina 	= $plugin_params->get('itp_mobile_retina','300');
		
		$itp_tablet 		= $plugin_params->get('itp_tablet','0');
		$itp_tablet_w 		= $plugin_params->get('itp_tablet_width','480');
		
		$itp_desktop		= $plugin_params->get('itp_desktop','0');
		$itp_desktop_w 		= $plugin_params->get('itp_desktop_width','768');
		
		$itp_tv 			= $plugin_params->get('itp_tv','0');
		$itp_tv_w 			= $plugin_params->get('itp_tv_width','890');

		$itp_opt_def 		= $plugin_params->get('itp_opt_def','0');
		$itp_opt_def_w		= $plugin_params->get('itp_opt_def_width','1600');


		$actual_size = getimagesize($img_original);


	


		//si la imagen mini es más pequeña que el ancho definido: false
		if($actual_size[0] <= $itp_mobile_w) return false;
		
		$img_fallback = $img_original;

		




		//ORIGINAL
		//if(plgSystemImgtopictureHelper::isAbsoluteUrl($img_original)){
		$img_original = self::_createThumbs(JString::trim($img_original, '"'),$ruta_thumb,$thumbs_path,'itp_orig');
		//}




		if (!file_exists($img_original)) {
			$img_original = self::$img_original_path;
		}



		

	
		//LAZY LOW 64
		$img_lazy_low = false;
		if(self::$itp_lazy){
			$img_lazy_low = self::_createThumbs(JString::trim($img_original, '"'),$ruta_thumb,$thumbs_path,$itp_lazy_low);
		}
		

		//MOBILE
		$img_low=false;
		if($itp_mobile){
			$img_low = self::_createThumbs(JString::trim($img_original, '"'),$ruta_thumb,$thumbs_path,$itp_mobile_w);
		}

		//RETINA
		$img_low_2x=false;
		if($itp_mobile_retina){
			$retina_w = $itp_mobile_w * 2;
			$img_low_2x = self::_createThumbs(JString::trim($img_original, '"'),$ruta_thumb,$thumbs_path,$retina_w);
		}
		

		//TABLET
		$img_med = false;
		if($itp_tablet){
			if($actual_size[0] <= $itp_tablet_w){
				$img_med = $img_original;
			}else{
				$img_med = self::_createThumbs(JString::trim($img_original, '"'),$ruta_thumb,$thumbs_path,$itp_tablet_w);
			}
		}


		//HI
		$img_hi=false;
		if($itp_desktop){
			if($actual_size[0] <= $itp_desktop_w){
				$img_hi = $img_original;
			}else{
				$img_hi = self::_createThumbs(JString::trim($img_original, '"'),$ruta_thumb,$thumbs_path,$itp_desktop_w);
			}
		}


		//EXTRA
		$img_extra=false;
		if($itp_tv){
			if($actual_size[0] <= $itp_tv_w){
				$img_extra = $img_original;
			}else{
				$img_extra = self::_createThumbs(JString::trim($img_original, '"'),$ruta_thumb,$thumbs_path,$itp_tv_w);
			}
		}


		//DEF
		$img_opt_def= false;
		if($itp_opt_def){
			if($actual_size[0] <= $itp_opt_def_w){
				$img_opt_def = $img_original;
			}else{
				$img_opt_def = self::_createThumbs(JString::trim($img_original, '"'),$ruta_thumb,$thumbs_path,$itp_opt_def_w);
			}
		}
 



		/*$resultado = array(
			'original'=>$img_original,
			'low'=>$img_low,
			'med'=>$img_med,
			'hi'=>$img_hi,
			'extra'=>$img_extra
		);*/


		//MODIFY IWIDTH TO LAZY LOAD

		if($iwidth == '') $iwidth = $actual_size[0];

/*
		var_dump($img_fallback.' | '.
			$img_original.' | '.
			$img_opt_def.' | '.
			$img_extra.' | '.
			$img_hi.' | '.
			$img_med.' | '.
			$img_low.' | '.
			$img_low_2x.' | '.
			$img_lazy_low.' | '.
			$alt, $itemprop.' | '.
			$class.' | '.
			$id.' | '.
			$title.' | '.
			$lang.' | '.
			$dir.' | '.
			$longdesc.' | '.
			$style.' | '.
			$iwidth.' | '.
			$iheight
		);
*/

		$resultado = $this->format_picture(
			$img_fallback,
			$img_original, 
			$img_opt_def, 
			$img_extra, 
			$img_hi, 
			$img_med, 
			$img_low,
			$img_low_2x,
			$img_lazy_low, 
			$alt, $itemprop, $class, $id, $title, $lang, $dir, $longdesc,$style,
			$iwidth,$iheight
		);


		return $resultado;
	}








	//create thumbs
	public static function _createThumbs($introimage_thumb,$ruta_thumb,$thumbs_path,$thumb_width)
	{

	

		self::addDebug("_createThumbs",'+++++++++++++++++++++ '.$thumb_width.' +++++++++++++++++++++');

		if(plgSystemImgtopictureHelper::isAbsoluteUrl($introimage_thumb)){	
			$ruta_img = $introimage_thumb;
		}else{
			$ruta_img = JPATH_BASE.'/'.$introimage_thumb;
		}
		
		$thumb_path_array = explode("/",$introimage_thumb);
		$thumb_name = end($thumb_path_array);

	
		//corto la extension
		$thumb_name_array = explode(".",$thumb_name);

		//inserto el denominador de tamanyo
		$thumb_name_array[0] = $thumb_name_array[0].'_'.$thumb_width;
		//inserto el punto a la extension
		$thumb_name_array[count($thumb_name_array)-1] = ".".$thumb_name_array[count($thumb_name_array)-1];
		//convierto el array en un string
		$thumb_name = implode($thumb_name_array);


		$thumb_name_right = plgSystemImgtopictureHelper::exttoWebp($thumb_name,self::$webp);

		
		if (file_exists($ruta_thumb.'/'.$thumb_name_right) && !self::$debug_images) {
			
			$resultado = 'cache/'.$thumbs_path.'/'.$thumb_name_right;
			self::addDebug("_createThumbs file exists",$resultado);

		}else{
			self::_createThumb_raw($ruta_img,$ruta_thumb,$thumb_name,$thumb_width);
			$resultado = 'cache/'.$thumbs_path.'/'.$thumb_name_right;
		}

		

		return $resultado;
	}



	public static function _createThumb_imagick($ruta_img,$ruta_thumb,$filename,$imgwidth){

			self::addDebug("_createThumb imagick" ,'PATH: '.$ruta_img.' <br> FINAL: cache/imagetopicture/'.$filename.' <br> WIDTH: '.$imgwidth.'  <br> '.self::$webp);

			$fparts = pathinfo($ruta_img);
			$ext = strtolower($fparts['extension']);

			/* if its not an image return false */
			if (!in_array($ext,array('gif','jpg','png','jpeg','webp'))){
				self::addDebug("_createThumb imagick" ,'image not supported');
				return false;
			}
			
			if(self::$debug) $draw = new ImagickDraw();

		    $img = new Imagick($ruta_img); 
		    $img->setImageResolution(72,72); 
			$img->resampleImage(72,72,imagick::FILTER_LANCZOS,1); 
			if($imgwidth != 'itp_orig'){
				$img->scaleImage($imgwidth,0); 
			}

			if(self::$debug) $draw = $img->annotateImage($draw, 10, 45, 0, $ext.' q:'.self::$png_quality.' w:'.$imgwidth);
				    
			$img->writeImage($ruta_thumb.'/'.$filename); 
		    
		    $img->destroy(); 
	} 


	public static function _createThumb_raw($src,$dest,$thumb_name,$desired_width = false, $desired_height = false)
	{
		self::addDebug("GD" ,'PATH: '.$src.' <br> FINAL: cache/imagetopicture/'.$thumb_name.' <br> WIDTH: '.$desired_width.'  <br> WEBP: '.self::$webp);


	    /*If no dimension for thumbnail given, return false */
		if (!$desired_height&&!$desired_width) return false;
		
	    $fparts = pathinfo($src);
	    $ext = strtolower($fparts['extension']);
	    /* if its not an image return false */
	    if (!in_array($ext,array('gif','jpg','png','jpeg','webp'))) return false;

	    /* read the source image */
	    if ($ext == 'gif')
	        $resource = imagecreatefromgif($src);
	    else if ($ext == 'png')
	        $resource = imagecreatefrompng($src);
	    else if ($ext == 'jpg' || $ext == 'jpeg')
			$resource = imagecreatefromjpeg($src);
		else if ($ext == 'webp' && self::$webp)
			$resource = imagecreatefromwebp($src);
	    
	    $width  = imagesx($resource);
	    $height = imagesy($resource);

		if($desired_width == 'itp_orig') $desired_width = $width;

	    /* find the "desired height" or "desired width" of this thumbnail, relative to each other, if one of them is not given  */
	    if(!$desired_height) $desired_height = floor($height*($desired_width/$width));
	    if(!$desired_width)  $desired_width  = floor($width*($desired_height/$height));
	  
	    /* create a new, "virtual" image */
	    $virtual_image = imagecreatetruecolor($desired_width,$desired_height);
	  
	    /* copy source image at a resized size */

	    //better quality
	    imagealphablending($virtual_image, false);
	    imagesavealpha($virtual_image, true);
	    $transparentindex = imagecolorallocatealpha($virtual_image, 255, 255, 255, 127);
	    imagecopyresampled($virtual_image,$resource,0,0,0,0,$desired_width,$desired_height,$width,$height);
	    // create the physical thumbnail image to its destination
	    // Use correct function based on the desired image type from $dest thumbnail source
	    $fparts = pathinfo($dest.'/'.$thumb_name);
	    $ext = strtolower($fparts['extension']);

	    //if dest is not an image type, default to jpg
	    if (!in_array($ext,array('gif','jpg','png','jpeg'))) $ext = 'jpg';
	    $dest = $fparts['dirname'].'/'.$fparts['filename'].'.'.$ext;
		$dest_webp = $fparts['dirname'].'/'.$fparts['filename'].'.webp';
	    
	    if ($ext == 'gif'){

			if(self::$webp) imagewebp($virtual_image,$dest_webp,self::$webp_quality);
			if(self::$itp_scale_all) imagegif($virtual_image,$dest);
			
	    }else if ($ext == 'png'){

			imagefill($virtual_image, 0, 0, $transparentindex);
			

			if(self::$debug){
				
				if(self::$webp) $string = 'webp/png q:'.self::$webp_quality.' dw:'.$desired_width.' dh:'.$desired_height;
				else $string = 'PNG q:'.self::$png_quality.' ds:'.$desired_width.' dh:'.$desired_height;
				if($desired_width == $width) $string.=' ORIG';

							
				$im = $virtual_image; // Path Images
				$color = ImageColorAllocate($im, 10, 0, 0); // Text Color
				$pxX = (Imagesx($im) - 6.5 * strlen($string))/2; // X
				$pxX = 10; // X
				$pxY = Imagesy($im)- 20; // Y
				if($desired_width == $width) $pxY = 20; // Y
				ImageString($im, 200, $pxX, $pxY, $string, $color);
				
				$virtual_image = $im;
			}

			if(self::$webp) imagewebp($virtual_image,$dest_webp,self::$webp_quality);
			if(self::$itp_scale_all) imagepng($virtual_image,$dest,self::$png_quality);
			imagedestroy($virtual_image);
	    }
	    else if ($ext == 'jpg' || $ext == 'jpeg'){

			if(self::$debug){
				if(self::$webp) $string = 'webp/jpg q:'.self::$webp_quality.' dw:'.$desired_width.' dh:'.$desired_height;
				else $string = 'JPG q:'.self::$jpg_quality.' ds:'.$desired_width.' dh:'.$desired_height;
				if($desired_width == $width) $string.=' ORIG';

				$im = $virtual_image; // Path Images
				$color = ImageColorAllocate($im, 10, 0, 0); // Text Color
				$pxX = (Imagesx($im) - 6.5 * strlen($string))/2; // X
				$pxY = Imagesy($im)- 20; // Y
				if($desired_width == $width) $pxY = 20; // Y
				ImageString($im,200, $pxX, $pxY, $string, $color);

				$virtual_image = $im;
			}

			if(self::$webp) imagewebp($virtual_image,$dest_webp, self::$webp_quality);
			if(self::$itp_scale_all) imagejpeg($virtual_image,$dest,self::$jpg_quality);
			
			imagedestroy($virtual_image);
	    }
	    
	    return array(
	        'width'     => $width,
	        'height'    => $height,
	        'new_width' => $desired_width,
	        'new_height'=> $desired_height,
			'dest'      => $dest,
			'format'    => $ext,
			'webp'		=> self::$webp,
	    );
	}

/* 	//CONTEXT
	public function checkContext($pluginParams, $context) {

		$cont_article = $pluginParams->get('itp_context_article','1');
		$cont_category = $pluginParams->get('itp_context_category','0');
		$cont_modcustom = $pluginParams->get('itp_context_modcustom','0');
		
		$trueorfalse = false;

		if($context=='com_content.article' && $cont_article){
			$trueorfalse=true;
		}
		if($context=='com_content.category' && $cont_category){
			$trueorfalse=true;
		}
		if($context=='mod_custom.content' && $cont_modcustom){
			$trueorfalse=true;
		}

		return $trueorfalse;

	} */

	

	public static function addDebug($func='', $msg='')
	{
		$cuenta = count(self::$debug_msg);
		self::$debug_msg[$cuenta]['func'] = $func;
		self::$debug_msg[$cuenta]['msg'] = $msg;
		self::$debug_msg[$cuenta]['microtime'] = microtime(true);
	}

	public static function getDebug()
	{
		$dbug = '<table class="table table-striped">';

		$anterior = 0;
		foreach(self::$debug_msg as $key => $val){
			$request_time = $val['microtime']-$_SERVER["REQUEST_TIME_FLOAT"];
			$actual = $request_time-$anterior;
			$dbug .= '<tr><td>'.$val['func'].'</td><td>'.$val['msg'].'</td><td>'.number_format($request_time,2).'</td><td><span class="badge badge-success">+'.number_format($actual,4).'</span></td></tr>';
			$anterior=$request_time;
		}

		$dbug .= '</table>';
		return $dbug;
	}


	public static function format_picture(
		$img_fallback,
		$img_original,
		$img_opt_def,
		$img_extra, 
		$img_hi,
		$img_med, 
		$img_low,
		$img_low_x2,
		$img_lazy_low, 
		$alt="", 
		$itemprop="", 
		$class="", 
		$id="", 
		$title="", 
		$lang="", 
		$dir="", 
		$longdesc="",
		$style="",
		$iwidth="",
		$iheight=""
	)
	{




		
		/*convert to absolute preventing nginx + sef error*/

		//FALLBACK
		if(!plgSystemImgtopictureHelper::isAbsoluteUrl($img_fallback)){
			$img_fallback = JURI::base().$img_fallback;
		}

		//ORIGINAL
		if(!plgSystemImgtopictureHelper::isAbsoluteUrl($img_original)){
			$img_original = JURI::base().$img_original;
		}
		

			//EXTRA
		if(!plgSystemImgtopictureHelper::isAbsoluteUrl($img_extra)){	
			$img_extra = JURI::base().$img_extra;
		}
		
		//default
		if(!plgSystemImgtopictureHelper::isAbsoluteUrl($img_opt_def)){
			$img_opt_def = JURI::base().$img_opt_def;
		}

		//hi
		if(!plgSystemImgtopictureHelper::isAbsoluteUrl($img_hi)){
			$img_hi = JURI::base().$img_hi;
		}

		//med
		if(!plgSystemImgtopictureHelper::isAbsoluteUrl($img_med)){
			$img_med = JURI::base().$img_med;
		}

		//lowx2
		if(!plgSystemImgtopictureHelper::isAbsoluteUrl($img_low_x2)){
			$img_low_x2 = JURI::base().$img_low_x2;
		}

		//low
		if(!plgSystemImgtopictureHelper::isAbsoluteUrl($img_low)){
			$img_low = JURI::base().$img_low;
		}

		//lazy low
		if($img_lazy_low){
			if(!plgSystemImgtopictureHelper::isAbsoluteUrl($img_lazy_low)){
				$img_lazy_low = JURI::base().$img_lazy_low;
			}
		}else{
			$img_lazy_low = '';
		}

		






		$plugin = JPluginHelper::getPlugin('system', 'imgtopicture');
		$plugin_params = new JRegistry($plugin->params);

		$article_img = $plugin_params->get('itp_article_images','0');

		$itp_mobile 	= $plugin_params->get('itp_mobile','0');
		$itp_mobile_w 	= $plugin_params->get('itp_mobile_width','300');
		$itp_tablet 	= $plugin_params->get('itp_tablet','0');
		$itp_tablet_w 	= $plugin_params->get('itp_tablet_width','480');
		$itp_desktop	= $plugin_params->get('itp_desktop','0');
		$itp_desktop_w 	= $plugin_params->get('itp_desktop_width','768');
		$itp_tv 		= $plugin_params->get('itp_tv','0');
		$itp_tv_w 		= $plugin_params->get('itp_tv_width','890');

		$itp_opt_def 	= $plugin_params->get('itp_opt_def','0');
		$itp_opt_def_w	= $plugin_params->get('itp_opt_def_width','1600');

		//old
		//$media_or_size 	= $plugin_params->get('itp_media_or_size','0');



		$itp_mobile_retina 	= $plugin_params->get('itp_mobile_retina','0');




		if($itp_opt_def){
			$actual_size = getimagesize($img_opt_def);
			//$img_default=$img_opt_def;
			$img_original = $img_opt_def;
		}else{
			$actual_size = getimagesize($img_original);		
		}

	





		/**********************************************/
		/*IMG to PICTURE Weeeeeeeeeeeeeeeeeeeeeeeeeeee*/
		/**********************************************/
		

		if($itp_mobile){
			$img_default=$img_low;
		}elseif($itp_tablet){
			$img_default=$img_med;
		}elseif($itp_desktop){
			$img_default=$img_hi;
		}elseif($itp_tv){

			if($itp_opt_def){
				$img_default=$img_opt_def;
			}else{
				$img_default=$img_original;
			}
		}


		$itp_lazy_data ='';
		$lazyclass='';
		if(self::$itp_lazy){
			$itp_lazy_data = 'data-';
			$lazyclass = 'lazyload';
		}


		$resultado ='<picture '.$id.' class="'.$class.'">';


				//low retina
				$retina = '';
				if($itp_mobile_retina){
					$retina = ', '.plgSystemImgtopictureHelper::exttoWebp($img_low_x2,self::$webp).' 2x';
				}
			

				$nextW = plgSystemImgtopictureHelper::getSizeAvailable($plugin_params,'extra');

				
				
				//EXTRA
				if($itp_opt_def && $actual_size[0] >= $nextW){
					
					if(self::$debug) $resultado .= '<!--itp_original ('.$actual_size[0].' >= '.$nextW.')-->';
					$resultado .= '<source 
						media="(min-width: '.$nextW .'px)" 
						'.$itp_lazy_data.'srcset="'.plgSystemImgtopictureHelper::exttoWebp($img_original,self::$webp).'">';
				}


				$nextW = plgSystemImgtopictureHelper::getSizeAvailable($plugin_params,'tv');
				//HI
 				if($itp_tv && $actual_size[0] >= $nextW){
					if(self::$debug) $resultado .= '<!--itp_tv ('.$actual_size[0].' >= '.$nextW.')-->';
					$resultado .= '<source 
						media="(min-width: '.$nextW.'px) and (max-width: '.$itp_tv_w.'px)" 
						'.$itp_lazy_data.'srcset="'.plgSystemImgtopictureHelper::exttoWebp($img_extra,self::$webp).'">';
				}

				$nextW = plgSystemImgtopictureHelper::getSizeAvailable($plugin_params,'desktop');
				//MED
	 			if($itp_desktop && $actual_size[0] >= $nextW){
					if(self::$debug) $resultado .= '<!--itp_desktop ('.$actual_size[0].' >= '.$nextW.')-->';
					$resultado .= '<source 
						media="(min-width: '.$nextW.'px) and (max-width: '.$itp_desktop_w.'px)" 
						'.$itp_lazy_data.'srcset="'.plgSystemImgtopictureHelper::exttoWebp($img_hi,self::$webp).'">';
				}

				$nextW = plgSystemImgtopictureHelper::getSizeAvailable($plugin_params,'tablet');
				//LOW
				if($itp_tablet && $actual_size[0] >= $nextW){
					if(self::$debug) $resultado .= '<!--itp_tablet ('.$actual_size[0].' >= '.$nextW.')-->';
					$resultado .= '<source 
						media="(min-width: '.$nextW.'px) and (max-width: '.$itp_tablet_w.'px)" 
						'.$itp_lazy_data.'srcset="'.plgSystemImgtopictureHelper::exttoWebp($img_med,self::$webp).$retina.'">';
				}


				//MIN
				if($itp_mobile && $actual_size[0] >= $itp_mobile_w){
					if(self::$debug) $resultado .= '<!--itp_mobile_w-->';
					$resultado .= '<source 
						media="(max-width: '.$itp_mobile_w.'px)" 
						'.$itp_lazy_data.'srcset="'.plgSystemImgtopictureHelper::exttoWebp($img_low,self::$webp).$retina.'">';
				}




			$src_low = '';
			if($img_lazy_low){
				$src_low = ' src="'.$img_lazy_low.'" width="'.$iwidth.'"';
			}
			

			$resultado .= '<img '.$id.' 
			 	class="'.$lazyclass.' '.$class.'" loading="lazy" width="'.$iwidth.'"';
				

				$src  = $itp_lazy_data.'src="'.$img_fallback.'"';
			    $srcset = $itp_lazy_data.'srcset="'.$img_fallback.'" ';
			

				$resultado .= $src_low; 
				$resultado .= $src; 
				$resultado .= $srcset; 
				$resultado .= $alt.' 
			    	'.$itemprop.' 
			    	'.$title.' 
			    	'.$lang.' 
			    	'.$dir.' 
			    	'.$longdesc.' 
			    	'.$style.'
			    	'.$iwidth.'
			    	'.$iheight.'
			>';


		
		$resultado .=	'</picture>';



	
		if(self::$debug) $resultado .= self::getDebug();

		self::$debug_msg = array();


		return $resultado;
	}







}
